import { fileURLToPath } from 'url'
import path from 'path';

import virtual from '@rollup/plugin-virtual';
import typescript from '@rollup/plugin-typescript';
import alias from '@rollup/plugin-alias';
import json from '@rollup/plugin-json';
import { nodeResolve } from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import replace from '@rollup/plugin-replace';
import terser from '@rollup/plugin-terser';
import gzipPlugin from 'rollup-plugin-gzip'
import { visualizer } from "rollup-plugin-visualizer";
import { babel } from '@rollup/plugin-babel';


const __dirname = fileURLToPath(new URL('.', import.meta.url))


/**
 * @type {import('rollup').RollupOptions}
 */
const config = {
	input: './src/index.ts',
	output: {
		file: 'bundle.js',
		format: 'umd',
        name: `vue3-sfc-loader`,

	},
    plugins: [
        nodeResolve({
            browser: false,
            exportConditions: ['module', 'node', 'require'],
            preferBuiltins: false,
        }),
        commonjs(),
        json(),
        replace({
            preventAssignment: true,
            values: {
                'process.env.NODE_ENV': JSON.stringify('production'),
                'process.env.NODE_DEBUG': 'undefined',
                'process.env.DEBUG': 'undefined',

                'process.env.BABEL_ENV': JSON.stringify('production'),
                'process.env.BABEL_TYPES_8_BREAKING': false,

                'process.env.VUE_ENV': JSON.stringify('client'),
                'global.process.env.VUE_ENV': JSON.stringify('client'),

				// config
				'process.env.GEN_SOURCEMAP': JSON.stringify(false),
				'process.env.VERSION': JSON.stringify(123),

            },
        }),

        virtual({
            debug: `
                export default function(scope) {
                    return (message) => {
                        console.log('debug', scope, message);
                    }
                }
            `,
            'noop.mjs': ``,
            fs: '',

        }),
        alias({
            entries: [
                { find: './createSFCModule', replacement: './createVue3SFCModule' },
                { find: './buble.js', replacement: path.resolve(__dirname, 'fakeBuble.mjs') }, // used by vue-template-es2015-compiler
                { find: 'chalk', replacement: path.resolve(__dirname, 'noopChalk.js') },
				
                ...'semver,@babel/highlight,source-map-js,@jridgewell,@vue/compiler-ssr,velocityjs,tinyliquid,liquid-node,jade,then-jade,dust,dustjs-helpers,dustjs-linkedin,swig,swig-templates,razor-tmpl,atpl,liquor,twig,ejs,eco,jazz,jqtpl,hamljs,hamlet,whiskers,haml-coffee,hogan.js,templayed,handlebars,underscore,lodash,pug,then-pug,qejs,walrus,mustache,just,ect,mote,toffee,dot,bracket-template,ractive,nunjucks,htmling,babel-core,plates,react-dom,react,arc-templates,vash,slm,marko,teacup,coffee-script,vm,squirrelly,twing'
                   .split(',')
				   .map(e => ({ find: new RegExp('^' + e + '.*$'), replacement: 'noop.mjs' })),

            ]
        }),
        typescript(), // beware: order is important !
        babel({
            babelHelpers: 'bundled',
            compact: true,
        }),
/*
        terser({
            maxWorkers: 4,
            mangle: true,
            compress: {
                passes: 2,
                drop_console: true,
            }
        }),
*/
        gzipPlugin(),
        visualizer({
            //template: 'network',
            emitFile: true,
            filename: "stats.html",
        }),
    ],
};

export default config;
